define(["exports", "../../../@polymer/polymer/polymer-element.js", "../../../@polymer/iron-resizable-behavior/iron-resizable-behavior.js"], function (_exports, _polymerElement, _ironResizableBehavior) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.AbsolutePositionStateManager = void 0;

  /**
   * Copyright 2018 The Pennsylvania State University
   * @license Apache-2.0, see License.md for full text.
   */
  // register globally so we can make sure there is only one
  window.AbsolutePositionStateManager = window.AbsolutePositionStateManager || {}; // request if this exists. This helps invoke the el existing in the dom
  // as well as that there is only one of them. That way we can ensure everything
  // is rendered through the same modal

  window.AbsolutePositionStateManager.requestAvailability = function () {
    if (!window.AbsolutePositionStateManager.instance) {
      window.AbsolutePositionStateManager.instance = document.createElement("absolute-position-state-manager");
      document.body.appendChild(window.AbsolutePositionStateManager.instance);
    }

    return window.AbsolutePositionStateManager.instance;
  };
  /**
   * `absolute-position-state-manager`
   * `A utility that manages the state of multiple a11y-media-players on a single page.`
   *
   * @microcopy - language worth noting:
   *  -
   *
   * @customElement
   * @polymer
   */


  var AbsolutePositionStateManager =
  /*#__PURE__*/
  function (_PolymerElement) {
    babelHelpers.inherits(AbsolutePositionStateManager, _PolymerElement);
    babelHelpers.createClass(AbsolutePositionStateManager, null, [{
      key: "tag",

      /* REQUIRED FOR TOOLING DO NOT TOUCH */

      /**
       * Store the tag name to make it easier to obtain directly.
       * @notice function name must be here for tooling to operate correctly
       */
      get: function get() {
        return "absolute-position-state-manager";
      } // properties available to the custom el for data binding

    }, {
      key: "properties",
      get: function get() {
        return {
          /**
           * Stores an array of all the players on the page.
           */
          els: {
            type: Array,
            value: []
          }
        };
      }
      /**
       * Makes sure there is a utility ready and listening for els.
       */

    }]);

    function AbsolutePositionStateManager() {
      var _this;

      babelHelpers.classCallCheck(this, AbsolutePositionStateManager);
      _this = babelHelpers.possibleConstructorReturn(this, babelHelpers.getPrototypeOf(AbsolutePositionStateManager).call(this));
      var root = babelHelpers.assertThisInitialized(_this); // sets the instance to the current instance

      if (!window.AbsolutePositionStateManager.instance) {
        window.AbsolutePositionStateManager.instance = babelHelpers.assertThisInitialized(_this);
        return babelHelpers.possibleConstructorReturn(_this, babelHelpers.assertThisInitialized(_this));
      }

      return _this;
    }
    /**
     * life cycle, el is afixed to the DOM
     * Makes sure there is a utility ready and listening for els.
     */


    babelHelpers.createClass(AbsolutePositionStateManager, [{
      key: "connectedCallback",
      value: function connectedCallback() {
        babelHelpers.get(babelHelpers.getPrototypeOf(AbsolutePositionStateManager.prototype), "connectedCallback", this).call(this);
      }
      /**
       * Loads el into array
       */

    }, {
      key: "loadElement",
      value: function loadElement(el) {
        var root = this;
        root.__on = root.__on !== undefined ? root.__on : false;
        root.els.push(el);
        root.positionElement(el);
        if (!root.__on) root.addEventListeners();
        root.__on = true;
      }
      /**
       * Unloads el from array
       */

    }, {
      key: "unloadElement",
      value: function unloadElement(el) {
        var root = this;
        root.els.filter(function (el) {
          return el === el;
        });
        root.__on = root.els > 0;
        if (!root.__on) root.removeEventListeners();
      }
      /**
       * Adds event listeners
       */

    }, {
      key: "addEventListeners",
      value: function addEventListeners() {
        var root = this;
        root.__timeout = false;
        root.updateElements();
        document.addEventListener("load", root.updateElements());
        window.addEventListener("resize", function () {
          clearTimeout(root.__timeout);
          root.__timeout = setTimeout(root.updateElements(), 250);
        });
        root.__observer = new MutationObserver(function (mutations) {
          root.checkMutations(mutations);
        });

        root.__observer.observe(document, {
          attributes: true,
          childList: true,
          subtree: true,
          characterData: true
        });
      }
      /**
       * Checks if there are any chances other than to
       * the element's position and updates accordioning.
       * This is needed so that positioning the elements
       * doesn't trigger an infinite loop of updates.
       *
       * @param {array} mutation records
       * @return {void}
       */

    }, {
      key: "checkMutations",
      value: function checkMutations(mutations) {
        var _this2 = this;

        var update = false;
        mutations.forEach(function (mutation) {
          if (update) return;
          update = update || !(mutation.type === "attributes" && mutation.attributeName === "style" && _this2.els.includes(mutation.target));
        });
        if (update) this.updateElements();
      }
      /**
       * Returns the target el that this el is anchored to. It is
       * either the el given by the `for` attribute, or the immediate parent
       * of the el.
       *
       * @param {object} element using absolute-position behavior
       * @return {object} target element for positioning
       */

    }, {
      key: "findTarget",
      value: function findTarget(el) {
        var selector = "#" + el.for,
            docQuery = document.querySelectorAll(selector).length === 1 ? document.querySelector(selector) : null,
            target = el.target || docQuery,
            ancestor = el;
        /**
         * Use `target` object if specified.
         * If not, query the document for elements with the id specified in the `for` attribute.
         * If there is more than one element that matches, find the closest matching element.
         */

        while (el.for !== undefined && target === null && ancestor !== null && ancestor !== document) {
          ancestor = ancestor.parentNode;
          if (ancestor.nodeType === 11) ancestor = ancestor.host;
          target = ancestor ? ancestor.querySelector(selector) : null;
        }

        return target;
      }
      /**
       * Removes event listeners
       * @return {void}
       */

    }, {
      key: "removeEventListeners",
      value: function removeEventListeners() {
        var root = this;
        document.removeEventListener("load", root.updateElements());
        window.removeEventListener("resize", function () {
          clearTimeout(root.__timeout);
          root.__timeout = setTimeout(root.updateElements(), 250);
        });

        if (root.__observer) {
          root.__observer.disconnect();
        }
      }
      /**
       * Updates position for all elements on the page.
       * @return {void}
       */

    }, {
      key: "updateElements",
      value: function updateElements() {
        var root = this;
        root.els.forEach(function (el) {
          root.positionElement(el);
        });
      }
      /**
       * Gets an updated position based on target.
       * @param {object} the element using absolute-position behavior
       * @return {void}
       */

    }, {
      key: "positionElement",
      value: function positionElement(el) {
        var target = this.findTarget(el);
        if (!target || !el.offsetParent) return;

        var offset = el.offset,
            parentRect = el.offsetParent.getBoundingClientRect(),
            targetRect = target.getBoundingClientRect(),
            elRect = el.getBoundingClientRect(),

        /**
         * place the element before the vertically?
         * @param {string} position as in "top", "left", "right", or "bottom"
         */
        vertical = function vertical(position) {
          return position !== "left" && position !== "right";
        },

        /**
         * place the element before the target?
         */
        before = function before(position) {
          return position === "left" || position === "top";
        },

        /**
         * fits the element within the parent's boundaries;
         * if element is larget than the parent,
         * it will be aligned where parent begins
         */
        fitToBounds = function fitToBounds() {
          var pos1 = vertical(el.position) ? "left" : "top",
              pos2 = vertical(el.position) ? "right" : "bottom",
              getRect = function getRect(rect) {
            return vertical(el.position) ? rect.width : rect.height;
          },
              coord = targetRect[pos1] - getRect(elRect) / 2 + getRect(targetRect) / 2,
              //works for left
          min = parentRect[pos1],
              max = parentRect[pos2] - getRect(elRect);

          return el.fitToVisibleBounds ? Math.max(min, Math.min(max, coord)) + "px" //Math.max(min, Math.min(max, coord)) + "px"
          : coord + "px";
        },

        /**
         * adds or subtracts the offset from the target based on a given postion
         */
        getCoord = function getCoord() {
          return el.position === "top" ? targetRect.top - elRect.height - offset + "px" : el.position === "left" ? targetRect.left - elRect.width - offset + "px" : targetRect[el.position] + offset + "px";
        },

        /**
         * determines if there is room for the element between
         * the parent and target in a given position;
         * if no room in any position it will return the original position
         */
        isFit = function isFit(position) {
          var size = vertical(position) ? elRect.height + offset : elRect.width + offset;
          return before(position) ? targetRect[position] - parentRect[position] > size : parentRect[position] - targetRect[position] > size;
        };

        var flip = el.fitToVisibleBounds !== false && !isFit(el.position),
            flipData = {
          top: ["bottom", "left", "right"],
          left: ["right", "top", "bottom"],
          bottom: ["top", "right", "left"],
          right: ["left", "bottom", "top"]
        };
        /*
         * fits the element according to specified postion,
         * or finds an alternative position that fits
         */

        if (flip && isFit(flipData[el.position][0])) {
          el.position = flipData[el.position][0];
        } else if (flip && isFit(flipData[el.position][1])) {
          el.position = flipData[el.position][1];
        } else if (flip && isFit(flipData[el.position][2])) {
          el.position = flipData[el.position][2];
        } else {
          el.style.position = "absolute";
          el.style.top = vertical(el.position) ? getCoord() : fitToBounds();
          el.style.left = vertical(el.position) ? fitToBounds() : getCoord(); //provide positions for el and target (in case furthor positioning adjustments are needed)

          el.__positions = {
            self: elRect,
            parent: parentRect,
            target: targetRect
          };
        }
      }
      /**
       * life cycle, el is removed from the DOM
       */

    }, {
      key: "disconnectedCallback",
      value: function disconnectedCallback() {
        this.removeEventListeners();
        babelHelpers.get(babelHelpers.getPrototypeOf(AbsolutePositionStateManager.prototype), "disconnectedCallback", this).call(this);
      }
    }]);
    return AbsolutePositionStateManager;
  }(_polymerElement.PolymerElement);

  _exports.AbsolutePositionStateManager = AbsolutePositionStateManager;
  window.customElements.define(AbsolutePositionStateManager.tag, AbsolutePositionStateManager);
});