define(["exports", "../../../simple-colors/simple-colors.js", "./haxcms-site-store.js", "../../../../mobx/lib/mobx.module.js", "../../../hax-body/lib/haxutils.js", "../../../../@polymer/polymer/lib/utils/async.js", "../../../../@polymer/polymer/lib/legacy/polymer.dom.js", "../../../../@polymer/polymer/lib/utils/render-status.js", "../../../../@polymer/polymer/lib/mixins/element-mixin.js"], function (_exports, _simpleColors, _haxcmsSiteStore, _mobxModule, _haxutils, _async, _polymerDom, _renderStatus, _elementMixin) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.HAXCMSThemeWiring = _exports.HAXCMSTheme = void 0;

  /**
   * Copyright 2019 The Pennsylvania State University
   * @license Apache-2.0, see License.md for full text.
   */

  /**
   * `HAXCMSTheme` mixin class to automatically apply HAXcms theme state
   * Typically an element will be extended from this and while not all,
   * many will want to customize the `contentContainer` property in order
   * to ensure the editable layer is correctly applied visually.
   */
  var HAXCMSTheme = function HAXCMSTheme(SuperClass) {
    return (
      /*#__PURE__*/
      function (_SuperClass) {
        babelHelpers.inherits(_class, _SuperClass);

        // leverage the wiring class element; this helps us clean things up smoothly later
        // while still keeping it abstract enough for direct usage in PolymerLegacy elements
        // as well as those wanting a custom integration methodology
        function _class() {
          var _this;

          babelHelpers.classCallCheck(this, _class);
          _this = babelHelpers.possibleConstructorReturn(this, babelHelpers.getPrototypeOf(_class).call(this));
          _this.__disposer = [];
          _this.HAXCMSThemeWiring = new HAXCMSThemeWiring(babelHelpers.assertThisInitialized(_this));
          return _this;
        }
        /**
         * This is a render function example. All new HAXcms capable themes need
         * to define a contentcontainer and a slot id wrapper. this allows HAXcms
         * to correctly target the area that will display the HAXeditor when in
         * edit-mode and correctly hide the editor when in normal content presentation.
         * static get template() {
         *  return html`
         *  <style include="simple-colors-shared-styles">
         *   :host {
         *     display: block;
         *     background-color: var(--haxcms-color, white);
         *   }
         *   :host([edit-mode]) #slot {
         *     display: none;
         *   }
         *  </style>
         *  <div id="contentcontainer">
         *    <div id="slot"><slot></slot></div>
         *  </div>`;
         *  }
         */


        babelHelpers.createClass(_class, [{
          key: "_colorChanged",
          value: function _colorChanged(newValue) {
            if (newValue) {
              this.hexColor = this._getHexColor(newValue);
            }
          }
          /**
           * Convert color name to HEX
           */

        }, {
          key: "_getHexColor",
          value: function _getHexColor(color) {
            // legacy support for materializeCSS names
            var name = color.replace("-text", "");
            var tmp = new _simpleColors.SimpleColors();

            if (tmp.colors[name]) {
              return tmp.colors[name][6];
            }

            return "#000000";
          }
          /**
           * notice edit changed, make sure we fake a resize because of that container flyout
           */

        }, {
          key: "_editModeChanged",
          value: function _editModeChanged(newValue, oldValue) {
            if (babelHelpers.typeof(oldValue) !== (typeof undefined === "undefined" ? "undefined" : babelHelpers.typeof(undefined))) {
              // ensure global is kept in sync
              _haxcmsSiteStore.store.editMode = newValue;

              _async.microTask.run(function () {
                // trick browser into thinking we just reized
                window.dispatchEvent(new Event("resize")); // forcibly update styles via css variables

                (0, _elementMixin.updateStyles)();
              });
            }
          }
          /**
           * private: Notice content container has changed
           */

        }, {
          key: "_contentContainerChanged",
          value: function _contentContainerChanged(newValue, oldValue) {
            // test that this hasn't been connected previously
            if (newValue && babelHelpers.typeof(oldValue) === (typeof undefined === "undefined" ? "undefined" : babelHelpers.typeof(undefined))) {
              this.HAXCMSThemeWiring.connect(this, newValue);
            } // previously connected, needs to change to new connection
            // this is an edge case at best...
            else if (newValue && oldValue) {
                this.HAXCMSThemeWiring.disconnect(this);
                this.HAXCMSThemeWiring.connect(this, newValue);
              } // no longer connected
              else if (oldValue && newValue == null) {
                  this.HAXCMSThemeWiring.disconnect(this);
                }
          }
        }, {
          key: "_locationChanged",
          value: function _locationChanged(newValue, oldValue) {
            if (!newValue || typeof newValue.route === "undefined") return;
            var location = newValue;
            var name = location.route.name;

            if (name === "home" || name === "404") {
              // if we are on the homepage then load the first item in the manifest
              // and set it active
              var firstItem = _haxcmsSiteStore.store.routerManifest.items.find(function (i) {
                return typeof i.id !== "undefined";
              });

              if (firstItem) {
                _haxcmsSiteStore.store.activeId = firstItem.id;
              }
            }
          }
          /**
           * Connect state and theme wiring
           */

        }, {
          key: "connectedCallback",
          value: function connectedCallback() {
            babelHelpers.get(babelHelpers.getPrototypeOf(_class.prototype), "connectedCallback", this).call(this);
            (0, _renderStatus.afterNextRender)(this, function () {
              var _this2 = this;

              // edge case, we just swapped theme faster then content loaded... lol
              setTimeout(function () {
                if ((0, _polymerDom.dom)(_this2).getEffectiveChildNodes().length === 0) {
                  var frag = document.createRange().createContextualFragment(_haxcmsSiteStore.store.activeItemContent);
                  (0, _polymerDom.dom)(_this2).appendChild(frag);
                }

                _async.microTask.run(function () {
                  // trick browser into thinking we just reized
                  window.dispatchEvent(new Event("resize")); // forcibly update styles via css variables

                  (0, _elementMixin.updateStyles)();
                });
              }, 50); // keep editMode in sync globally

              (0, _mobxModule.autorun)(function (reaction) {
                _this2.editMode = (0, _mobxModule.toJS)(_haxcmsSiteStore.store.editMode);

                _this2.__disposer.push(reaction);
              }); // logged in so we can visualize things differently as needed

              (0, _mobxModule.autorun)(function (reaction) {
                _this2.isLoggedIn = (0, _mobxModule.toJS)(_haxcmsSiteStore.store.isLoggedIn);

                _this2.__disposer.push(reaction);
              }); // store disposer so we can clean up later

              (0, _mobxModule.autorun)(function (reaction) {
                var __manifest = (0, _mobxModule.toJS)(_haxcmsSiteStore.store.manifest);

                if (__manifest && (0, _haxutils.varExists)(__manifest, "title")) {
                  document.title = __manifest.title;
                }

                if (__manifest && (0, _haxutils.varExists)(__manifest, "metadata.theme.variables.cssVariable")) {
                  // json outline schema changed, allow other things to react
                  // fake way of forcing an update of these items
                  var ary = __manifest.metadata.theme.variables.cssVariable.replace("--simple-colors-default-theme-", "").split("-");

                  ary.pop(); // simple colors "accent color" property

                  _this2.accentColor = ary.join("-");
                  var color = (0, _haxutils.varGet)(__manifest, "metadata.theme.variables.cssVariable", null); // fallback if color wasn't set via css var

                  if (color == null) {
                    color = (0, _haxutils.varGet)(__manifest, "metadata.theme.variables.hexCode", "#ff0074");
                  } else {
                    color = "var(".concat(color, ")");
                  } // set this directly instead of messing w/ accentColor


                  document.body.style.setProperty("--haxcms-color", color);
                }

                _this2.__disposer.push(reaction);
              });
              (0, _mobxModule.autorun)(function (reaction) {
                _this2._location = _haxcmsSiteStore.store.location;

                _this2.__disposer.push(reaction);
              }); // we don't have a content container, establish one

              if (babelHelpers.typeof(this.contentContainer) === (typeof undefined === "undefined" ? "undefined" : babelHelpers.typeof(undefined))) {
                this.contentContainer = this.shadowRoot.querySelector("#contentcontainer");
              }
            });
          }
          /**
           * Disconnect the wiring for the theme and clean up state
           */

        }, {
          key: "disconnectedCallback",
          value: function disconnectedCallback() {
            babelHelpers.get(babelHelpers.getPrototypeOf(_class.prototype), "disconnectedCallback", this).call(this); // remove our content container var which will disconnect the wiring

            delete this.contentContainer; // clean up state

            for (var i in this.__disposer) {
              this.__disposer[i].dispose();
            }
          }
          /**
           * Correctly reset state and dispatch event to notify of active item change
           */

        }, {
          key: "resetActive",
          value: function resetActive() {
            window.history.pushState(null, null, _haxcmsSiteStore.store.location.baseUrl);
            window.dispatchEvent(new PopStateEvent("popstate"));
            this.dispatchEvent(new CustomEvent("haxcms-active-item-changed", {
              bubbles: true,
              composed: true,
              cancelable: true,
              detail: {}
            }));
          }
        }], [{
          key: "properties",
          get: function get() {
            var props = {
              /**
               * Class for the color
               */
              hexColor: {
                type: String
              },

              /**
               * Color class work to apply
               */
              color: {
                type: String,
                reflectToAttribute: true,
                observer: "_colorChanged"
              },

              /**
               * editting state for the page
               */
              editMode: {
                type: Boolean,
                reflectToAttribute: true,
                notify: true,
                value: false,
                observer: "_editModeChanged"
              },

              /**
               * editting state for the page
               */
              isLoggedIn: {
                type: Boolean,
                reflectToAttribute: true,
                notify: true,
                value: false
              },

              /**
               * DOM node that wraps the slot
               */
              contentContainer: {
                type: Object,
                notify: true,
                observer: "_contentContainerChanged"
              },

              /**
               * location as object
               */
              _location: {
                type: Object,
                observer: "_locationChanged"
              }
            };

            if (babelHelpers.get(babelHelpers.getPrototypeOf(_class), "properties", this)) {
              props = Object.assign(props, babelHelpers.get(babelHelpers.getPrototypeOf(_class), "properties", this));
            }

            return props;
          }
        }]);
        return _class;
      }(SuperClass)
    );
  };
  /**
   * `HAXCMSThemeWiring` streamline hooking themes up to HAXCMS
   * Directly invoking this class is not advised unless
   * the mixin class `HAXCMSTheme` integration needs modified beyond the norm
   */


  _exports.HAXCMSTheme = HAXCMSTheme;

  var HAXCMSThemeWiring =
  /*#__PURE__*/
  function () {
    function HAXCMSThemeWiring(element) {
      var load = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
      babelHelpers.classCallCheck(this, HAXCMSThemeWiring);

      if (load) {
        // @todo may want to set this to sessionStorage instead...
        if (window.localStorage.getItem("HAXCMSSystemData") == null) {
          window.localStorage.setItem("HAXCMSSystemData", JSON.stringify({}));
        }
      }
    }
    /**
     * connect the theme and see if we have an authoring experience to inject correctly
     */


    babelHelpers.createClass(HAXCMSThemeWiring, [{
      key: "connect",
      value: function connect(element, injector) {
        window.addEventListener("haxcms-active-item-changed", this._activeItemUpdate.bind(element));
        window.addEventListener("haxcms-edit-mode-changed", this._globalEditChanged.bind(element));
        window.addEventListener("haxcms-trigger-update", this._triggerUpdate.bind(element)); // inject the tools to allow for an authoring experience
        // ensuring they are loaded into the correct theme

        _haxcmsSiteStore.store.cmsSiteEditorAvailability();

        _haxcmsSiteStore.store.cmsSiteEditor.instance.appElement = element;
        _haxcmsSiteStore.store.cmsSiteEditor.instance.appendTarget = injector;

        _haxcmsSiteStore.store.cmsSiteEditor.instance.appendTarget.appendChild(_haxcmsSiteStore.store.cmsSiteEditor.instance);
      }
      /**
       * detatch element events from whats passed in
       */

    }, {
      key: "disconnect",
      value: function disconnect(element) {
        window.removeEventListener("haxcms-active-item-changed", this._activeItemUpdate.bind(element));
        window.removeEventListener("haxcms-edit-mode-changed", this._globalEditChanged.bind(element));
        window.removeEventListener("haxcms-trigger-update", this._triggerUpdate.bind(element)); // need to unplug this so that the new theme can pick it up.

        document.body.appendChild(_haxcmsSiteStore.store.cmsSiteEditorAvailability());
      }
      /**
       * Global edit state changed
       */

    }, {
      key: "_globalEditChanged",
      value: function _globalEditChanged(e) {
        this.editMode = e.detail;
      }
      /**
       * HAXcms: Active item has been updated
       */

    }, {
      key: "_activeItemUpdate",
      value: function _activeItemUpdate(e) {
        var newValue = e.detail;

        if (newValue && babelHelpers.typeof(newValue.id) !== (typeof undefined === "undefined" ? "undefined" : babelHelpers.typeof(undefined))) {
          // dispatch to the store
          _haxcmsSiteStore.store.activeId = newValue.id; // dispatch to everything else caring

          var evt = new CustomEvent("json-outline-schema-active-item-changed", {
            bubbles: true,
            composed: true,
            cancelable: true,
            detail: newValue
          });
          this.dispatchEvent(evt); // update title as a simple nicity

          if (babelHelpers.typeof(newValue.title) !== (typeof undefined === "undefined" ? "undefined" : babelHelpers.typeof(undefined))) {
            document.title = _haxcmsSiteStore.store.routerManifest.title + " - " + newValue.title;
          } else {
            document.title = _haxcmsSiteStore.store.routerManifest.title;
          }
        } else {
          document.title = _haxcmsSiteStore.store.routerManifest.title;
        }
      }
      /**
       * Generic event to ensure that the active item change is noticed
       */

    }, {
      key: "_triggerUpdate",
      value: function _triggerUpdate(e) {
        this.dispatchEvent(new CustomEvent("haxcms-active-item-changed", {
          bubbles: true,
          composed: true,
          cancelable: true,
          detail: {}
        }));
      }
    }]);
    return HAXCMSThemeWiring;
  }();

  _exports.HAXCMSThemeWiring = HAXCMSThemeWiring;
});