define(["../polymer/polymer-legacy.js", "../iron-a11y-keys-behavior/iron-a11y-keys-behavior.js", "../paper-radio-button/paper-radio-button.js", "../iron-menu-behavior/iron-menubar-behavior.js", "../iron-selector/iron-selectable.js", "../polymer/lib/legacy/polymer-fn.js", "../polymer/lib/utils/html-tag.js"], function (_polymerLegacy, _ironA11yKeysBehavior, _paperRadioButton, _ironMenubarBehavior, _ironSelectable, _polymerFn, _htmlTag) {
  "use strict";

  function _templateObject_97005e20e53211e9a0cfefb808989d41() {
    var data = babelHelpers.taggedTemplateLiteral(["\n    <style>\n      :host {\n        display: inline-block;\n      }\n\n      :host ::slotted(*) {\n        padding: var(--paper-radio-group-item-padding, 12px);\n      }\n    </style>\n\n    <slot></slot>\n"]);

    _templateObject_97005e20e53211e9a0cfefb808989d41 = function _templateObject_97005e20e53211e9a0cfefb808989d41() {
      return data;
    };

    return data;
  }

  /**
  Material design: [Radio
  button](https://www.google.com/design/spec/components/selection-controls.html#selection-controls-radio-button)
  
  `paper-radio-group` allows user to select at most one radio button from a set.
  Checking one radio button that belongs to a radio group unchecks any
  previously checked radio button within the same group. Use
  `selected` to get or set the selected radio button.
  
  The <paper-radio-buttons> inside the group must have the `name` attribute
  set.
  
  Example:
  
      <paper-radio-group selected="small">
        <paper-radio-button name="small">Small</paper-radio-button>
        <paper-radio-button name="medium">Medium</paper-radio-button>
        <paper-radio-button name="large">Large</paper-radio-button>
      </paper-radio-group>
  
  Radio-button-groups can be made optional, and allow zero buttons to be selected:
  
      <paper-radio-group selected="small" allow-empty-selection>
        <paper-radio-button name="small">Small</paper-radio-button>
        <paper-radio-button name="medium">Medium</paper-radio-button>
        <paper-radio-button name="large">Large</paper-radio-button>
      </paper-radio-group>
  
  See <a href="paper-radio-button">paper-radio-button</a> for more
  information about `paper-radio-button`.
  
  
  Custom property | Description | Default
  ----------------|-------------|----------
  `--paper-radio-group-item-padding` | The padding of the item | `12px`
  
  @group Paper Elements
  @element paper-radio-group
  @demo demo/index.html
  */
  (0, _polymerFn.Polymer)({
    _template: (0, _htmlTag.html)(_templateObject_97005e20e53211e9a0cfefb808989d41()),
    is: 'paper-radio-group',
    behaviors: [_ironMenubarBehavior.IronMenubarBehavior],

    /** @private */
    hostAttributes: {
      role: 'radiogroup'
    },
    properties: {
      /**
       * Fired when the radio group selection changes.
       *
       * @event paper-radio-group-changed
       */

      /**
       * Overriden from Polymer.IronSelectableBehavior
       */
      attrForSelected: {
        type: String,
        value: 'name'
      },

      /**
       * Overriden from Polymer.IronSelectableBehavior
       */
      selectedAttribute: {
        type: String,
        value: 'checked'
      },

      /**
       * Overriden from Polymer.IronSelectableBehavior
       */
      selectable: {
        type: String,
        value: 'paper-radio-button'
      },

      /**
       * If true, radio-buttons can be deselected
       */
      allowEmptySelection: {
        type: Boolean,
        value: false
      }
    },

    /**
     * Selects the given value.
     */
    select: function select(value) {
      var newItem = this._valueToItem(value);

      if (newItem && newItem.hasAttribute('disabled')) {
        return;
      }

      if (this.selected) {
        var oldItem = this._valueToItem(this.selected);

        if (this.selected == value) {
          // If deselecting is allowed we'll have to apply an empty selection.
          // Otherwise, we should force the selection to stay and make this
          // action a no-op.
          if (this.allowEmptySelection) {
            value = '';
          } else {
            if (oldItem) oldItem.checked = true;
            return;
          }
        }

        if (oldItem) oldItem.checked = false;
      }

      _ironSelectable.IronSelectableBehavior.select.apply(this, [value]);

      this.fire('paper-radio-group-changed');
    },
    _activateFocusedItem: function _activateFocusedItem() {
      this._itemActivate(this._valueForItem(this.focusedItem), this.focusedItem);
    },
    _onUpKey: function _onUpKey(event) {
      this._focusPrevious();

      event.preventDefault();

      this._activateFocusedItem();
    },
    _onDownKey: function _onDownKey(event) {
      this._focusNext();

      event.preventDefault();

      this._activateFocusedItem();
    },
    _onLeftKey: function _onLeftKey(event) {
      _ironMenubarBehavior.IronMenubarBehaviorImpl._onLeftKey.apply(this, arguments);

      this._activateFocusedItem();
    },
    _onRightKey: function _onRightKey(event) {
      _ironMenubarBehavior.IronMenubarBehaviorImpl._onRightKey.apply(this, arguments);

      this._activateFocusedItem();
    }
  });
});