define(["exports", "require", "../../../../../@polymer/polymer/polymer-element.js", "../haxcms-site-store.js", "../../../../jwt-login/jwt-login.js"], function (_exports, _require, _polymerElement, _haxcmsSiteStore, _jwtLogin) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.HAXCMSBackendDemo = void 0;
  _require = babelHelpers.interopRequireWildcard(_require);

  /**
   * Copyright 2018 The Pennsylvania State University
   * @license Apache-2.0, see License.md for full text.
   */

  /**
   * `haxcms-backend-demo`
   * `a simple element to check for and fetch JWTs`
   *
   * @demo demo/index.html
   *
   * @microcopy - the mental model for this element
   * - jwt - a json web token which is an encrypted security token to talk
   */
  class HAXCMSBackendDemo extends _polymerElement.PolymerElement {
    /**
     * Store the tag name to make it easier to obtain directly.
     * @notice function name must be here for tooling to operate correctly
     */
    static get tag() {
      return "haxcms-backend-demo";
    } // render function


    static get template() {
      return _polymerElement.html`
      <jwt-login
        auto
        id="jwt"
        url="[[jwtLoginLocation]]"
        url-logout="[[jwtLogoutLocation]]"
        jwt="{{jwt}}"
      ></jwt-login>
    `;
    }

    static get properties() {
      return {
        /**
         * Location of what endpoint to hit for
         */
        jwtLoginLocation: {
          type: String,
          value: function () {
            if (window.appSettings) {
              return window.appSettings.login;
            }
          }
        },

        /**
         * Location of what endpoint to hit for logging out
         */
        jwtLogoutLocation: {
          type: String,
          value: function () {
            if (window.appSettings) {
              return window.appSettings.logout;
            }
          }
        },

        /**
         * JSON Web token, it'll come from a global call if it's available
         */
        jwt: {
          type: String,
          observer: "_jwtChanged"
        }
      };
    }
    /**
     * created life cycle
     */


    constructor() {
      super();
      document.body.addEventListener("jwt-token", this._jwtTokenFired.bind(this)); // this makes it easier to debug things and directly modify the store for
      // development purposes

      window.HAXcmsStore = _haxcmsSiteStore.store;
    }

    _jwtChanged(newValue) {
      console.log(newValue);
    }
    /**
     * detached life cycle
     */


    disconnectedCallback() {
      document.body.removeEventListener("jwt-token", this._jwtTokenFired.bind(this));
      super.disconnectedCallback();
    }
    /**
     * JWT token fired, let's capture it
     */


    _jwtTokenFired(e) {
      this.jwt = e.detail;
      _haxcmsSiteStore.store.jwt = this.jwt;
    }
    /**
     * Attached life cycle
     */


    connectedCallback() {
      super.connectedCallback();

      try {
        new Promise((res, rej) => _require.default(["../haxcms-site-editor.js"], res, rej)).then(e => {
          // validate availability
          _haxcmsSiteStore.store.cmsSiteEditorAvailability();

          _haxcmsSiteStore.store.cmsSiteEditor.instance.jwt = this.jwt;
          _haxcmsSiteStore.store.cmsSiteEditor.instance.method = "GET";
          _haxcmsSiteStore.store.cmsSiteEditor.instance.saveNodePath = window.appSettings.saveNodePath;
          _haxcmsSiteStore.store.cmsSiteEditor.instance.saveManifestPath = window.appSettings.saveManifestPath;
          _haxcmsSiteStore.store.cmsSiteEditor.instance.saveOutlinePath = window.appSettings.saveOutlinePath;
          _haxcmsSiteStore.store.cmsSiteEditor.instance.getNodeFieldsPath = window.appSettings.getNodeFieldsPath;
          _haxcmsSiteStore.store.cmsSiteEditor.instance.getSiteFieldsPath = window.appSettings.getSiteFieldsPath;
          _haxcmsSiteStore.store.cmsSiteEditor.instance.createNodePath = window.appSettings.createNodePath;
          _haxcmsSiteStore.store.cmsSiteEditor.instance.deleteNodePath = window.appSettings.deleteNodePath;
          _haxcmsSiteStore.store.cmsSiteEditor.instance.publishSitePath = window.appSettings.publishSitePath;
          _haxcmsSiteStore.store.cmsSiteEditor.instance.syncSitePath = window.appSettings.syncSitePath;
          _haxcmsSiteStore.store.cmsSiteEditor.instance.revertSitePath = window.appSettings.revertSitePath;
          _haxcmsSiteStore.store.cmsSiteEditor.instance.getUserDataPath = window.appSettings.getUserDataPath;
          _haxcmsSiteStore.store.cmsSiteEditor.instance.appStore = window.appSettings.appStore;
        }, e => {
          //import failed
          console.log(e);
        });
      } catch (err) {// error in the event this is a double registration
      }
    }

  }

  _exports.HAXCMSBackendDemo = HAXCMSBackendDemo;
  window.customElements.define(HAXCMSBackendDemo.tag, HAXCMSBackendDemo);
});