/**
 * Copyright 2019 The Pennsylvania State University
 * @license Apache-2.0, see License.md for full text.
 */
import { html, PolymerElement } from "../../../../../@polymer/polymer/polymer-element.js";
import { store } from "../../core/haxcms-site-store.js";
import { autorun, toJS } from "../../../../../mobx/lib/mobx.module.js";
/**
 * `site-footer`
 * `A basic site footer`
 *
 * @customElement
 * @polymer
 * @demo demo/index.html
 */

class SiteFooter extends PolymerElement {
  /**
   * Store the tag name to make it easier to obtain directly.
   * @notice function name must be here for tooling to operate correctly
   */
  static get tag() {
    return "site-footer";
  }

  constructor() {
    super();
    this.__disposer = [];
    import("../../../../license-element/license-element.js");
  } // render function


  static get template() {
    return html`
      <style>
        :host {
          display: block;
          transition: 0.2s opacity linear;
          opacity: 1;
        }
        :host([edit-mode]) {
          opacity: 0.2;
          pointer-events: none;
        }
      </style>
      <div class="wrapper">
        <license-element
          title="[[siteTitle]]"
          creator="[[manifest.author]]"
          source="[[manifest.domain]]"
          license="[[manifest.license]]"
        >
        </license-element>
      </div>
    `;
  }

  static get properties() {
    return {
      siteTitle: {
        type: String
      },
      manifest: {
        type: Object
      },
      editMode: {
        type: Boolean,
        reflectToAttribute: true
      }
    };
  }

  connectedCallback() {
    super.connectedCallback();
    autorun(reaction => {
      this.manifest = toJS(store.manifest);

      this.__disposer.push(reaction);
    });
    autorun(reaction => {
      this.editMode = toJS(store.editMode);

      this.__disposer.push(reaction);
    });
    autorun(reaction => {
      this.siteTitle = toJS(store.siteTitle);

      this.__disposer.push(reaction);
    });
  }

  disconnectedCallback() {
    // clean up state
    for (var i in this.__disposer) {
      this.__disposer[i].dispose();
    }

    super.disconnectedCallback();
  }

}

window.customElements.define(SiteFooter.tag, SiteFooter);
export { SiteFooter };